"""TMY3 data set library: thin wrapper around TMY csv files.

Examples:
    >>> sum([int(i['GHI (W/m^2)']) for i in data('724666')])/365./1000.
    4.438213698630137

    >>> round(total('724666', 'DNI (W/m^2)')/365.,2)
    5.11

"""
import csv
# Copyright (C) 2015 Nathan Charles
#
# This program is free software. See terms in LICENSE file.

import datetime
import os
import logging
logger = logging.getLogger(__name__)
from .tools import download
from . import env

# path to tmy3 data
# default = ~/tmp3/

try:
    os.listdir(os.environ['HOME'] + '/tmy3')
except OSError:
    try:
        os.mkdir(os.environ['HOME'] + '/tmy3')
    except IOError:
        pass


def tmybasename(usaf):
    """Basename for USAF base.

    Args:
        usaf (str): USAF code

    Returns:
        (str)
    """
    url_file = open(env.SRC_PATH + '/tmy3.csv')
    for line in url_file.readlines():
        if line.find(usaf) is not -1:
            return line.rstrip().partition(',')[0]


def _tmy_url(usaf):
    """tmy3 url."""
    tmybase = tmybasename(usaf)
    url = "http://rredc.nrel.gov/solar/old_data/nsrdb/1991-2005/data/tmy3/"
    # url = "http://rredc.nrel.gov/solar/old_data/nsrdb/1991-2005/data/tmy3.20080820/"
    return url + tmybase


def strptime(string, timezone=0):
    """custom strptime - necessary because of 24:00 end of day labeling."""
    year = int(string[6:10])
    month = int(string[0:2])
    day = int(string[3:5])
    hour = int(string[11:13])
    minute = int(string[14:16])
    return datetime.datetime(year, month, day) + \
        datetime.timedelta(hours=hour, minutes=minute) - \
        datetime.timedelta(hours=timezone)


def normalize_date(tmy_date, year):
    """change TMY3 date to an arbitrary year.

    Args:
        tmy_date (datetime): date to mangle.
        year (int): desired year.

    Returns:
        (None)
    """
    month = tmy_date.month
    day = tmy_date.day - 1
    hour = tmy_date.hour
    # hack to get around 24:00 notation
    if month is 1 and day is 0 and hour is 0:
        year = year + 1
    return datetime.datetime(year, month, 1) + \
        datetime.timedelta(days=day, hours=hour, minutes=0)


class data(object):

    """TMY3 iterator object.

    Atributes:
        latiude (float)
        longitude (float)
        tz (float)
    """

    def __init__(self, usaf):
        """initialize.

        Args:
            usaf (str)

        Returns:
            (object)
        """
        filename = env.WEATHER_DATA_PATH + '/' + usaf + 'TYA.csv'
        self.csvfile = None
        try:
            self.csvfile = open(filename)
        except IOError:
            logger.info("%s not found", filename)
            download(_tmy_url(usaf), filename)
            self.csvfile = open(filename)
        logging.debug('opened %s', self.csvfile.name)
        header = self.csvfile.readline().split(',')
        self.tmy_data = csv.DictReader(self.csvfile)
        self.latitude = float(header[4])
        self.longitude = float(header[5])
        self.tz = float(header[3])

    def __iter__(self):
        """iterate."""
        return self

    def next(self):
        """iterate."""
        record = self.tmy_data.next()
        _sd = record['Date (MM/DD/YYYY)'] + ' ' + record['Time (HH:MM)']
        record['utc_datetime'] = strptime(_sd, self.tz)
        record['datetime'] = strptime(_sd)
        return record

    def __del__(self):
        """del."""
        self.csvfile.close()


def total(usaf, field='GHI (W/m^2)'):
    """total annual insolation, defaults to GHI."""
    running_total = 0
    usafdata = data(usaf)
    for record in usafdata:
        running_total += float(record[field])
    return running_total/1000.

if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    import doctest
    doctest.testmod()



"""Forecast IO API."""
import json
import urllib2
import os
import datetime

APIKEY = os.getenv('FORECASTIO')
if not APIKEY:
    print "WARNING: forecast.io key not set."
    print "Realtime weather data not availible."


def data(place):
    """get forecast data."""
    lat, lon = place
    url = "https://api.forecast.io/forecast/%s/%s,%s?solar" % (APIKEY, lat,
                                                               lon)
    w_data = json.loads(urllib2.urlopen(url).read())
    return w_data


def mangle(data_point):
    """mangle data into expected format."""
    temp_dict = {}
    temp_dict.update(data_point)
    temp_dict['utc_datetime'] = \
        datetime.datetime.utcfromtimestamp(temp_dict['time'])
    if 'solar' in data_point:
        temp_dict['GHI (W/m^2)'] = data_point['solar']['ghi']
        temp_dict['DNI (W/m^2)'] = data_point['solar']['dni']
        temp_dict['DHI (W/m^2)'] = data_point['solar']['dhi']
        temp_dict['ETR (W/m^2)'] = data_point['solar']['etr']
        del temp_dict['solar']
    else:
        temp_dict['GHI (W/m^2)'] = 0.0
        temp_dict['DNI (W/m^2)'] = 0.0
        temp_dict['DHI (W/m^2)'] = 0.0
        temp_dict['ETR (W/m^2)'] = 0.0
    return temp_dict

# these functions should probably be class methods


def hourly(place):
    """return data as list of dicts with all data filled in."""
    # time in utc?
    lat, lon = place
    url = "https://api.forecast.io/forecast/%s/%s,%s?solar" % (APIKEY, lat,
                                                               lon)
    w_data = json.loads(urllib2.urlopen(url).read())
    hourly_data = w_data['hourly']['data']
    mangled = []
    for i in hourly_data:
        mangled.append(mangle(i))
    return mangled


def current(place):
    """return data as list of dicts with all data filled in."""
    lat, lon = place
    url = "https://api.forecast.io/forecast/%s/%s,%s?solar" % (APIKEY, lat,
                                                               lon)
    w_data = json.loads(urllib2.urlopen(url).read())
    currently = w_data['currently']
    return mangle(currently)

if __name__ == '__main__':
    LAT, LON = (40., -78.0)
    URL = "https://api.forecast.io/forecast/%s/%s,%s" % (APIKEY, LAT, LON)
    print URL
    print hourly((LAT, LON))
    print current((LAT, LON))



"""Download GFS grib2 files.

Example:
    Download latest to ~/gfs/[GFS TIMESTAMP]

    >>> gfs.download(datetime.datetime.now(),gfs.pgrb2)

"""
# Copyright (C) 2015 Nathan Charles
#
# This program is free software. See terms in LICENSE file.

import urllib2
import datetime
import csv
import os

DATA_PATH = os.environ['HOME'] + "/gfs"


def _verify_path(path):
    """look for path and create it if doesn't exist."""
    try:
        os.listdir(path)
    except OSError:
        os.mkdir(path)

_verify_path(DATA_PATH)


def baseurl(gfs_timestamp, filename):
    """build url."""
    url = 'http://nomads.ncep.noaa.gov/pub/data/nccf/com/gfs/prod/gfs.%s/'\
          '%s' % (gfs_timestamp, filename)
    return url


def _join(segments):
    """simply list by joining adjacent segments."""
    new = []
    start = segments[0][0]
    end = segments[0][1]
    for i in range(len(segments)-1):
        if segments[i+1][0] != segments[i][1]:
            new.append((start, end))
            start = segments[i+1][0]
        end = segments[i+1][1]
    new.append((start, end))
    return new


def _filter_messages(messages, products=None, levels=None):
    """filter messages for desired products and levels."""
    if products is None:
        products = []
    if levels is None:
        levels = []
    segments = []
    bounds = len(messages)
    for i, message in enumerate(messages):
        if (message[3] in products or len(products) == 0) and \
                (message[4] in levels or len(levels) == 0):
            start = int(message[1])
            if i == (bounds - 1):
                end = None
            else:
                end = int(messages[i+1][1])
            segments.append((start, end))
    return _join(segments)


def _download_segments(filename, url, segments):
    """download segments into a single file."""
    gribfile = open(filename, 'w')
    for start, end in segments:
        req = urllib2.Request(url)
        req.add_header('User-Agent',
                       'caelum/0.1 +https://github.com/nrcharles/caelum')
        if end:
            req.headers['Range'] = 'bytes=%s-%s' % (start, end)
        else:
            req.headers['Range'] = 'bytes=%s' % (start)
        opener = urllib2.build_opener()
        gribfile.write(opener.open(req).read())
    gribfile.close()


def sflux(closest, offset):
    """sflux dataset naming convention."""
    return 'gfs.t%02dz.sfluxgrbf%02d.grib2' % (closest, offset)


def pgrb2(closest, offset):
    """pgrb2 1 degree dataset naming convention."""
    # gfs.t12z.pgrb2.1p00.f000
    return 'gfs.t%02dz.pgrb2.1p00.f%03d' % (closest, offset)


def download(timestamp, dataset, path=None, products=None,
             levels=None, offset=0):
    """save GFS grib file to DATA_PATH.

    Args:
        dataset(function): naming convention function.  eg. pgrb2
        timestamp(datetime): ???
        path(str): if None defaults to DATA_PATH
        products(list): TMP, etc. if None downloads all.
        layers(list): surface, etc. if None downloads all.
        offset(int): should be multiple of 3
    """
    if path is None:
        path = DATA_PATH
    closest = timestamp.hour//6*6
    filename = dataset(closest, offset)
    gfs_timestamp = '%s%02d' % (timestamp.strftime('%Y%m%d'), closest)

    url = baseurl(gfs_timestamp, filename)
    index = url + '.idx'
    messages = message_index(index)
    segments = _filter_messages(messages, products, levels)
    dl_path = path + '/%s/' % gfs_timestamp
    _verify_path(dl_path)
    _download_segments(path + filename, url, segments)


def message_index(index_url):
    """get message index of components for urllib2.

    Args:
        url(string):

    Returns:
        list: messages
    """
    idx = csv.reader(urllib2.urlopen(index_url), delimiter=':')
    messages = []
    for line in idx:
        messages.append(line)
    return messages

if __name__ == '__main__':
    START = datetime.datetime.now()
    PRODUCTS = ['TCDC', 'SNOWC', 'TMP', 'ALBDO', 'UGRD', 'VGRD']
    LEVELS = ['surface', 'entire atmosphere (considered as a single layer)',
              'low cloud layer', '1 hybrid level']

    for j in range(5):
        download(START, sflux, products=PRODUCTS, levels=LEVELS, offset=j*3)



#!/usr/bin/env python
# Copyright (C) 2015 Nathan Charles
#
# This program is free software. See terms in LICENSE file.
"""Global EERE Weather Data Sources wrapper.

Examples:
    Annual GHI for STATION_CODE in Watts
    >>> sum([int(i['GHI (W/m^2)']) for i in EPWdata('418830')])
    1741512

    Minimum Design Temperature
    >>> minimum('418830')
    8.8

    2% Max Design Temperature
    >>> twopercent('418830')
    34.3

"""

from __future__ import absolute_import
from __future__ import print_function
import csv
import datetime
import pytz
import re
from geopy import geocoders
from .tools import download_extract
from . import env
import logging
logger = logging.getLogger(__name__)

DATA_EXTENTIONS = {'SWERA': 'epw',
                   'CSWD': 'epw',
                   'CWEC': 'epw',
                   'ETMY': 'epw',
                   'IGDG': 'epw',
                   'IMGW': 'epw',
                   'INETI': 'epw',
                   'ISHRAE': 'epw',
                   'ITMY': 'epw',
                   'IWEC': 'epw',
                   'KISR': 'epw',
                   'MSI': 'epw',
                   'NIWA': 'epw',
                   'RMY': 'epw',
                   'SWEC': 'epw',
                   'TMY': 'epw',
                   'TMY2': 'epw',
                   'TMY3': 'epw',
                   'IWEC': 'epw',
                   'stat': 'stat',
                   'ddy': 'ddy'}


def _muck_w_date(record):
    """muck with the date because EPW starts counting from 1 and goes to 24."""
    # minute 60 is actually minute 0?
    temp_d = datetime.datetime(int(record['Year']), int(record['Month']),
                               int(record['Day']), int(record['Hour']) % 24,
                               int(record['Minute']) % 60)
    d_off = int(record['Hour'])//24   # hour 24 is actually hour 0
    if d_off > 0:
        temp_d += datetime.timedelta(days=d_off)
    return temp_d


def _eere_url(station_code):
    """build EERE EPW url for station code."""
    baseurl = 'http://apps1.eere.energy.gov/buildings/energyplus/weatherdata/'
    return baseurl + _station_info(station_code)['url']


def _station_info(station_code):
    """filename based meta data for a station code."""
    url_file = open(env.SRC_PATH + '/eere.csv')
    for line in csv.DictReader(url_file):
        if line['station_code'] == station_code:
            return line
    raise KeyError('Station not found')


def _basename(station_code, fmt=None):
    """region, country, weather_station, station_code, data_format, url."""
    info = _station_info(station_code)
    if not fmt:
        fmt = info['data_format']
    basename = '%s.%s' % (info['url'].rsplit('/', 1)[1].rsplit('.', 1)[0],
                          DATA_EXTENTIONS[fmt])
    return basename


def twopercent(station_code):
    """Two percent high design temperature for a location.

    Degrees in Celcius

    Args:
        station_code (str): Weather Station Code

    Returns:
        float degrees Celcius
    """
    # (DB=>MWB) 2%, MaxDB=
    temp = None
    try:
        fin = open('%s/%s' % (env.WEATHER_DATA_PATH,
                              _basename(station_code, 'ddy')))
        for line in fin:
            value = re.search("""2%, MaxDB=(\\d+\\.\\d*)""", line)
            if value:
                temp = float(value.groups()[0])
    except IOError:
        pass

    if not temp:
        # (DB=>MWB) 2%, MaxDB=
        try:
            fin = open('%s/%s' % (env.WEATHER_DATA_PATH,
                                  _basename(station_code, 'stat')))
            flag = 0
            tdata = []
            for line in fin:
                if line.find('2%') is not -1:
                    flag = 3
                if flag > 0:
                    tdata.append(line.split('\t'))
                    flag -= 1
            temp = float(tdata[2][5].strip())
        except IOError:
            pass
    if temp:
        return temp
    else:
        raise Exception("Error: 2% High Temperature not found")


def minimum(station_code):
    """Extreme Minimum Design Temperature for a location.

    Degrees in Celcius

    Args:
        station_code (str): Weather Station Code

    Returns:
        float degrees Celcius
    """
    temp = None
    fin = None
    try:
        fin = open('%s/%s' % (env.WEATHER_DATA_PATH,
                              _basename(station_code, 'ddy')))
    except IOError:
        logger.info("File not found")
        download_extract(_eere_url(station_code))
        fin = open('%s/%s' % (env.WEATHER_DATA_PATH,
                              _basename(station_code, 'ddy')))
    for line in fin:
        value = re.search('Max Drybulb=(-?\\d+\\.\\d*)', line)
        if value:
            temp = float(value.groups()[0])
    if not temp:
        try:
            fin = open('%s/%s' % (env.WEATHER_DATA_PATH,
                                  _basename(station_code, 'stat')))
            for line in fin:
                if line.find('Minimum Dry Bulb') is not -1:
                    return float(line[37:-1].split('\xb0')[0])
        except IOError:
            pass
    if temp:
        return temp
    else:
        raise Exception("Error: Minimum Temperature not found")


class EPWdata(object):

    """EPW weather generator.

    Attributes:
        lat (float): latitude in degrees
        lon (float): lonitude in degrees

    """

    def __init__(self, station_code, DST=False):
        """Data for a weather station.

        Args:
            station_code (str): Station code of weather station
            DST (bool): Weather timestands in daylight savings. Default False
        """
        filename = env.WEATHER_DATA_PATH + '/' + _basename(station_code)
        self.csvfile = None
        try:
            self.csvfile = open(filename)
        except IOError:
            logger.info("File not found")
            download_extract(_eere_url(station_code))
            self.csvfile = open(filename)
        logging.debug('opened %s', self.csvfile.name)
        fieldnames = ["Year", "Month", "Day", "Hour", "Minute", "DS",
                      "Dry-bulb (C)", "Dewpoint (C)", "Relative Humidity",
                      "Pressure (Pa)", "ETR (W/m^2)", "ETRN (W/m^2)",
                      "HIR (W/m^2)", "GHI (W/m^2)", "DNI (W/m^2)",
                      "DHI (W/m^2)", "GHIL (lux)", "DNIL (lux)", "DFIL (lux)",
                      "Zlum (Cd/m2)", "Wdir (degrees)", "Wspd (m/s)",
                      "Ts cover", "O sky cover", "CeilHgt (m)",
                      "Present Weather", "Pw codes", "Pwat (cm)",
                      "AOD (unitless)", "Snow Depth (cm)",
                      "Days since snowfall"]
        station_meta = self.csvfile.readline().split(',')
        self.station_name = station_meta[1]
        self.CC = station_meta[3]
        self.station_fmt = station_meta[4]
        self.station_code = station_meta[5]
        self.lat = station_meta[6]
        self.lon = station_meta[7]
        self.TZ = float(station_meta[8])
        self.ELEV = station_meta[9]
        self.DST = DST

        if self.DST:
            geocoder = geocoders.GoogleV3()
            self.local_tz = pytz.timezone(geocoder.timezone((self.lat,
                                                             self.lon)).zone)
        dummy = ""
        for _ in range(7):
            dummy += self.csvfile.readline()
        self.epw_data = csv.DictReader(self.csvfile, fieldnames=fieldnames)

    def __iter__(self):
        """iterate."""
        return self

    def next(self):
        """Weather data record.

        Yields:
            dict
        """
        record = self.epw_data.next()
        local_time = _muck_w_date(record)
        record['datetime'] = local_time
        # does this fix a specific data set or a general issue?
        if self.DST:
            localdt = self.local_tz.localize(record['datetime'])
            record['utc_datetime'] = localdt.astimezone(pytz.UTC)
        else:
            record['utc_datetime'] = local_time - \
                datetime.timedelta(hours=self.TZ)

        return record
        # 'LOCATION,BEEK,-,NLD,IWEC Data,063800,50.92,5.78,1.0,116.0'

    def __del__(self):
        """clean up open files."""
        self.csvfile.close()

if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)
    import doctest
    doctest.testmod()



"""wrapper around NOAA GFS api"""
import logging
logger = logging.getLogger(__name__)

import urllib2
import xml.etree.ElementTree as ET
from scipy.interpolate import interp1d
import datetime

def forecast(place, series=True):
    """NOAA weather forecast for a location"""
    lat, lon = place
    url = "http://graphical.weather.gov/xml/SOAP_server/ndfdXMLclient.php?" + \
            "whichClient=NDFDgen&" + "lat=%s&lon=%s&" % (lat, lon) + \
            "Unit=e&temp=temp&wspd=wspd&sky=sky&wx=wx&rh=rh&" + \
            "product=time-series&Submit=Submit"
    logger.debug(url)
    res = urllib2.urlopen(url).read()
    root = ET.fromstring(res)
    time_series = [(i.text) for i in \
            root.findall('./data/time-layout')[0].iterfind('start-valid-time')]
    logger.debug(res)
    #knots to mph
    wind_speed = [eval(i.text)*1.15 for i in \
            root.findall('./data/parameters/wind-speed')[0].iterfind('value')]
    cloud_cover = [eval(i.text)/100.0 for i in \
            root.findall('./data/parameters/cloud-amount')[0].iterfind('value')]
    temperature = [eval(i.text) for i in \
            root.findall('./data/parameters/temperature')[0].iterfind('value')]
    if not series:
        return {'cloudCover':cloud_cover[0], \
                'temperature':temperature[0], \
                'windSpeed':wind_speed[0], \
                'start-valid-time':time_series[0]}
    else:
        return {'cloudCover':cloud_cover, \
                'temperature':temperature, \
                'windSpeed':wind_speed, \
                'start-valid-time':time_series}

def _str_time(string):
    """unused fuction?"""
    fmt = '%Y-%m-%dT%H:%M:00'
    return datetime.datetime.strptime(string[0:19], fmt)

def _cast_float(temp_dt):
    """returns utc timestamp"""
    if type(temp_dt) == str:
        fmt = '%Y-%m-%dT%H:%M:00'
        base_dt = temp_dt[0:19]
        tz_offset = eval(temp_dt[19:22])
        temp_dt = datetime.datetime.strptime(base_dt, fmt) - \
                datetime.timedelta(hours=tz_offset)
    return (temp_dt - datetime.datetime(1970, 1, 1)).total_seconds()

def herp_derp_interp(place):
    """simple interpolation of GFS forecast"""
    lat, lon = place
    #begin=2014-02-14T00%3A00%3A00&end=2018-02-22T00%3A00%3A00
    fmt = '%Y-%m-%dT00:00:00'
    fmt = '%Y-%m-%dT%H:%M:00'
    begin = (datetime.datetime.now()-datetime.timedelta(hours=12)).strftime(fmt)
    #end=(datetime.datetime.now()+datetime.timedelta(hours=48)).strftime(fmt)
    url = "http://graphical.weather.gov/xml/SOAP_server/ndfdXMLclient.php?" + \
            "whichClient=NDFDgen&lat=%s&lon=%s&" % (lat, lon) + \
            "Unit=e&temp=temp&wspd=wspd&sky=sky&wx=wx&rh=rh&" + \
            "product=time-series&begin=%s&end=2018-02-22T00:00:00" % begin + \
            "&Submit=Submit"""
    res = urllib2.urlopen(url).read()
    root = ET.fromstring(res)

    time_series = [_cast_float(i.text) for i in \
            root.findall('./data/time-layout')[0].iterfind('start-valid-time')]
    #knots to mph
    wind_speed = [eval(i.text)*1.15 for i in \
            root.findall('./data/parameters/wind-speed')[0].iterfind('value')]
    cloud_cover = [eval(i.text)/100.0 for i in \
            root.findall('./data/parameters/cloud-amount')[0].iterfind('value')]
    temperature = [eval(i.text) for i in \
            root.findall('./data/parameters/temperature')[0].iterfind('value')]

    ws_interp = interp1d(time_series, wind_speed, kind='cubic')
    cc_interp = interp1d(time_series, cloud_cover, kind='cubic')
    t_interp = interp1d(time_series, temperature, kind='cubic')
    start_date = datetime.datetime.utcfromtimestamp(time_series[0])

    series = []
    for i in range(48):
        try:
            temp_dict = {}
            forecast_dt = start_date + datetime.timedelta(hours=i)
            temp_dict['utc_datetime'] = forecast_dt
            temp_dict['windSpeed'] = ws_interp(_cast_float(forecast_dt)).item()
            temp_dict['temperature'] = t_interp(_cast_float(forecast_dt)).item()
            temp_dict['cloudCover'] = cc_interp(_cast_float(forecast_dt)).item()
            series.append(temp_dict)
        except:
            pass
    return series

if __name__ == '__main__':
    from solpy import geo
    PLACE = geo.zip_coordinates('17603')
    #print forecast(place)
    print herp_derp_interp(PLACE)
    #print wind_speed, len(wind_speed)
    #print cloudCover, len(cloudCover)
    #print temperature, len(temperature)



from tools import parse_noaa_line, closest_eere
from tools import eere_station
from geopy.distance import great_circle


import csv

with open('./inswo-stns.txt') as foo:
    foo.readline()
    foo.readline()
    while foo:
        b = parse_noaa_line(foo.readline())
        try:
            s = eere_station(b['station_code'])
            c, n = closest_eere(b['LAT'],b['LON'])
            if b['station_code'] != c:
                #print 'different station', b, c, n
                d = great_circle((b['LAT'],b['LON']),(s['latitude'],s['longitude']))
                if d > 100:
                    print b['station_code'], c, s['station_code'], s['weather_station'], n, d
                #print b
        except:
            #print 'not found'
            pass



"""Unit tests."""
import unittest


class BaseEERETest(unittest.TestCase):

    """Annual GHI."""

    def runTest(self):
        """GHI."""
        from caelum import eere
        SCODE = '063800'
        annual_ghi = sum([float(i['GHI (W/m^2)'])
                         for i in eere.EPWdata(SCODE)])
        SCODE = '724666'
        annual_ghi2 = sum([float(i['GHI (W/m^2)'])
                          for i in eere.EPWdata(SCODE)])
        print annual_ghi2
        self.assertAlmostEquals(annual_ghi, 985289)


class eereDesign(unittest.TestCase):

    """Design constrains."""

    def runTest(self):
        """Min/Max temps."""
        from caelum import eere
        SCODE = '063800'
        self.assertAlmostEquals(eere.minimum(SCODE), -10.2)
        self.assertAlmostEquals(eere.twopercent(SCODE), 25.4)
        SCODE = '724666'
        self.assertAlmostEquals(eere.minimum(SCODE), -22.9)
        self.assertAlmostEquals(eere.twopercent(SCODE), 30.2)

if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(BaseEERETest)
    unittest.TextTestRunner(verbosity=2).run(suite)



import csv
'USAF,Site Name,State,Latitude,Longitude,TZ,Elev,Class,Pool'
'region,country,weather_station,station_code,data_format,url'

from geopy import geocoders
geocoder = geocoders.GoogleV3()

f = csv.DictReader(open('./notfound.csv'))

for i in f:
    try:
        pl_name =  i['weather_station'].replace('.',' ')
        #pl_name = i['country'] +' '+  i['weather_station'].replace('.',' ')
        loc = geocoder.geocode(pl_name)
        tz = geocoder.timezone(loc.point)
        b = [i['station_code'], i['weather_station'], i['country'],
            loc.latitude, loc.longitude, tz.zone, loc.altitude, 'IV','?']
        print ','.join([str(i) for i in b])
    except Exception as e:
        print e




import unittest
import unit

class CoverageTest(unittest.TestCase):
    def test_module(self):
        from coverage import coverage
        cov = coverage()
        cov.start()
        suite = unittest.TestLoader().loadTestsFromTestCase(unit.BaseEERETest)
        unittest.TextTestRunner(verbosity=2).run(suite)
        cov.stop()
        r = cov.report()
        self.assertAlmostEquals(r, 985289)

if __name__ == '__main__':
    unittest.main()



"""Enviroment."""
import os

WEATHER_DATA_PATH = os.environ['HOME'] + "/weather_data"
SRC_PATH = os.path.dirname(os.path.abspath(__file__))

try:
    os.listdir(WEATHER_DATA_PATH)
except OSError:
    try:
        os.mkdir(WEATHER_DATA_PATH)
    except IOError:
        pass



# Copyright (C) 2012 Nathan Charles
#
# This program is free software. See terms in LICENSE file.
"""look at NSRDB historical data"""
import csv
import datetime
import os
import numpy as np
from solpy import tools

# path to data defaults to cwd unless NCDCDATA enviromental variable is set
CWD = os.getcwd()
PATH = [os.getenv('NCDCDATA', CWD)]


def strptime(string, timezone=0):
    """necessary because of 24:00 end of day labeling"""
    year = int(string[0:4])
    month = int(string[5:7])
    day = int(string[8:10])
    hour = int(string[-5:-3])
    minute = int(string[-2:])
    ts = datetime.datetime(year, month, day) + \
        datetime.timedelta(hours=hour, minutes=minute) - \
        datetime.timedelta(hours=timezone)
    return ts


def normalize_date(tmy_date, year):
    """change TMY3 date to an arbitrary year"""
    year = year
    month = tmy_date.month
    day = tmy_date.day - 1
    hour = tmy_date.hour
    minute = 0
    # hack to get around 24:00 notation
    if month is 1 and day is 0 and hour is 0:
        year += 1
    return datetime.datetime(year, month, 1) + \
        datetime.timedelta(days=day, hours=hour, minutes=minute)


class Data():
    """data generator"""
    def __init__(self, usaf, year=2010):
        basefile = "NSRDB_StationData_%s0101_%s1231_%s.csv" % \
            (year, year, usaf)
        filename = tools.find_file(basefile, PATH)
        self.usaf_info = geo.station_info(usaf)
        self.timezone = int(self.usaf_info['TZ'])
        self.place = (float(self.usaf_info['Latitude']),
                      float(self.usaf_info['Longitude']))
        self.csvfile = None
        try:
            self.csvfile = open(filename)
        except:
            print "File not found"
        self.tmy_data = csv.DictReader(self.csvfile)

    def __iter__(self):
        return self

    def next(self):
        t = self.tmy_data.next()
        sd = t['YYYY-MM-DD'] + ' ' + t['HH:MM (LST)']
        t['utc_datetime'] = strptime(sd, self.timezone)
        t['datetime'] = strptime(sd)
        return t

    def __del__(self):
        self.csvfile.close()


def monthly(usaf, year, field='GHI (W/m^2)'):
    """monthly insolation"""
    m = []
    lastm = 1
    usafdata = Data(usaf, year)
    t = 0
    for r in usafdata:
        r['GHI (W/m^2)'] = r['Glo Mod (Wh/m^2)']
        r['DHI (W/m^2)'] = r['Dif Mod (Wh/m^2)']
        r['DNI (W/m^2)'] = r['Dir Mod (Wh/m^2)']
        if r['datetime'].month != lastm:
            m.append(t/1000.)
            t = 0
            lastm = r['utc_datetime'].month
        t += float(r[field])
    return m


def report(usaf):
    """generate report for usaf base"""
    fig = plt.figure()
    ax = fig.add_subplot(111)
    station_info = geo.station_info(usaf)
    y = {}
    for i in range(1991, 2011):
        monthData = monthly(usaf, i)
        t = sum(monthData)
        y[i] = t
        print t
    tmy3tot = tmy3.total(usaf)
    average = sum([v for k, v in y.items()])/20.
    s = sorted(y.items(), key=lambda t: t[1])
    o = sorted(y.items(), key=lambda t: t[0])
    twohigh = s[-1][1] + s[-2][1]
    twolow = s[0][1] + s[1][1]
    mintol = 1-twolow/2./average
    plustol = twohigh/2./average-1
    txt = ""
    txt += "%s\n" % station_info['Site Name']
    txt += 'TMY3/hist: %s/' % int(round(tmy3tot))
    txt += '%s\n' % int(round(average))
    txt += "high/low av: %s/" % int(round(twohigh/2.))
    txt += "%s\n" % int(round(twolow/2.))
    txt += "+%s/-%s%% " % (round(plustol*100, 0), round(mintol*100, 0))
    txt += "(-%s%% of TMY3)" % round((1-twolow/2./tmy3tot)*100, 0)
    print txt
    x = np.array([k for k, v in o])
    y = np.array([v for k, v in o])

    rx = x[1:]
    ry = [(v + y[i+1])/2 for i, v in enumerate(y[:-1])]
    fit = pylab.polyfit(x, y, 3)
    fit_fn = pylab.poly1d(fit)
    f = interp1d(x, y, kind='cubic')
    f2 = interp1d(rx, ry, kind='cubic')
    xnew = np.linspace(min(x), max(x), 200)
    x2 = np.linspace(min(rx), max(rx), 200)
    # ax.plot(x,y)
    ax.plot(xnew, f(xnew), label="Annual GHI")
    ax.plot(xnew, fit_fn(xnew), label='trendline')
    ax.plot(x2, f2(x2), label='2 Year Ave')
    ax.plot([min(x), max(x)], [tmy3tot, tmy3tot], linestyle='--')
    leg = plt.legend(title=txt, loc=4, fancybox=True)
    leg.get_frame().set_alpha(0.5)
    # fig.text(min(x),min(y)-min(y)*.1,txt)
    # fig.text(.1,.1,txt)
    plt.tight_layout()
    fig.savefig('%s_annual_GHI.pdf' % (usaf), format='pdf')

if __name__ == "__main__":
    from solpy import geo
    from solpy import tmy3
    tilt = 32.0
    import matplotlib
    matplotlib.use('Agg')
    import matplotlib.pyplot as plt
    from scipy.interpolate import interp1d
    # from scipy import stats
    import pylab
    usaf_stations = []
    usaf_stations.append('912850')
    usaf_stations.append('725115')  # MDT
    usaf_stations.append('724290')  # ,DAYTON INTERNATIONAL AIRPORT
    usaf_stations.append('724070')  # ,ATLANTIC CITY INTL AP
    usaf_stations.append('724060')  # ,BALTIMORE BLT-WASHNGTN INT'L
    #
    for station in usaf_stations:
        report(station)



"""Helper functions."""
import csv
import os
import urllib2
import zipfile
import tempfile
import logging
from . import env
from geopy.distance import great_circle

SRC_PATH = os.path.dirname(os.path.abspath(__file__))

SLON = {'W': -1.0,
        'E': 1.0}
SLAT = {'N': 1.0,
        'S': -1.0}
logger = logging.getLogger(__name__)


def download(url, filename):
    """download and extract file."""
    logger.info("Downloading %s", url)
    request = urllib2.Request(url)
    request.add_header('User-Agent',
                       'caelum/0.1 +https://github.com/nrcharles/caelum')
    opener = urllib2.build_opener()
    local_file = open(filename, 'w')
    local_file.write(opener.open(request).read())
    local_file.close()


def download_extract(url):
    """download and extract file."""
    logger.info("Downloading %s", url)
    request = urllib2.Request(url)
    request.add_header('User-Agent',
                       'caelum/0.1 +https://github.com/nrcharles/caelum')
    opener = urllib2.build_opener()
    with tempfile.TemporaryFile(suffix='.zip', dir=env.WEATHER_DATA_PATH) \
            as local_file:
        logger.debug('Saving to temporary file %s', local_file.name)
        local_file.write(opener.open(request).read())
        compressed_file = zipfile.ZipFile(local_file, 'r')
        logger.debug('Extracting %s', compressed_file)
        compressed_file.extractall(env.WEATHER_DATA_PATH)
        local_file.close()


def _mlat(tlat):
    nlat = SLAT[tlat[-1:]] * float(tlat[0:-2])/10.
    return nlat


def _mlon(tlon):
    nlon = SLON[tlon[-1:]] * float(tlon[0:-2])/10.
    return nlon


def parse_noaa_line(line):
    """Parse NOAA stations.

    This is an old list, the format is:

    NUMBER NAME & STATE/COUNTRY                     LAT   LON     ELEV (meters)

    010250 TROMSO                             NO  6941N 01855E    10
    """
    station = {}
    station['station_name'] = line[7:51].strip()
    station['station_code'] = line[0:6]
    station['CC'] = line[55:57]
    station['ELEV(m)'] = int(line[73:78])
    station['LAT'] = _mlat(line[58:64])
    station['LON'] = _mlon(line[65:71])
    station['ST'] = line[52:54]
    return station


def closest_noaa(latitude, longitude):
    """Find closest station from the old list."""
    with open(env.SRC_PATH + '/inswo-stns.txt') as index:
        index.readline()  # header
        index.readline()  # whitespace
        min_dist = 9999
        station_name = ''
        station_name = ''
        for line in index:
            try:
                i = parse_noaa_line(line)
                new_dist = great_circle((latitude, longitude),
                                        (float(i['LAT']),
                                         float(i['LON']))).miles

            except:
                logger.error(line)
                raise IOError('Inventory Issue')

            if new_dist < min_dist:
                min_dist = new_dist
                station_name = i['station_name']
                station_code = i['station_code']
        index.close()
        return station_code, station_name
    raise KeyError('station not found')


def closest_eere(latitude, longitude):
    """Find closest station from the new(er) list.

    Warning: There may be some errors with smaller non US stations.

    Args:
        latitude (float)
        longitude (float)

    Returns:
        tuple (station_code (str), station_name (str))

    """
    with open(env.SRC_PATH + '/eere_meta.csv') as eere_meta:
        stations = csv.DictReader(eere_meta)
        d = 9999
        station_code = ''
        station_name = ''
        for station in stations:
            new_dist = great_circle((latitude, longitude),
                                    (float(station['latitude']),
                                     float(station['longitude']))).miles
            if new_dist <= d:
                d = new_dist
                station_code = station['station_code']
                station_name = station['weather_station']
        return station_code, station_name
    raise KeyError('station not found')


def eere_station(station_code):
    """Station information.

    Args:
        station_code (str): station code.

    Returns (dict): station information
    """
    with open(env.SRC_PATH + '/eere_meta.csv') as eere_meta:
        stations = csv.DictReader(eere_meta)
        for station in stations:
            if station['station_code'] == station_code:
                return station
    raise KeyError('station not found')

if __name__ == '__main__':
    print(closest_noaa(52.208364, 5.320569))
    print eere_station('163120')



import csv
'USAF,Site Name,State,Latitude,Longitude,TZ,Elev,Class,Pool'
'region,country,weather_station,station_code,data_format,url'

from geopy import geocoders
geocoder = geocoders.GoogleV3()

f = csv.DictReader(open('./eere.csv'))
for i in f:
    try:
        if i['region'] != '4' and i['region'] != 'problem':
            loc = geocoder.geocode(i['country'] +', ' + i['weather_station'])
            tz = geocoder.timezone(loc.point)
            b = [i['station_code'], i['weather_station'], i['country'],
                loc.latitude, loc.longitude, tz.zone, loc.altitude, 'IV']
            print ', '.join([str(i) for i in b])
    except Exception as e:
        print e

